;;; humanfns.el --- Human68K kanji / binary fileio support functions.
;;;   partly taken from dos-fns.el

;; Copyright (C) 1994 Free Software Foundation, Inc. / MINOURA Makoto

;; This file is part of GNU Emacs for Human68K.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Code:

(defvar file-name-buffer-file-type-alist	; it's too heavy for X68000
  '(
    ("[:/].*config.sys$" . nil)		; config.sys text
    ("\\.elc$" . t)			; emacs stuff
    ("\\.\\(o\\|x\\|r\\|a\\|l\\|sys\\)$" . t)
					; Human68K stuff
    ("\\.\\(arc\\|zip\\|pak\\|lzh\\|zoo\\)$" . t)
					; Packers
    ("\\.\\(elc\\|tar\\|z\\|Z\\|gz\\|taz\\|tgz\\)$" . t)
					; Unix stuff
    ("[:/]tags$" . t)
					; Emacs TAGS file
    )
  "*Alist for distinguishing text files from binary files.
Each element has the form (REGEXP . TYPE), where REGEXP is matched
against the file name, and TYPE is nil for text, t for binary.")

;;(defvar binary-file-extentions		; from Nemacs68k by icam.
;;  '(".elc" ".o" ".x" ".r" ".a" ".arc" ".zip" ".pak" ".lzh" ".zoo"
;;    ".elc" ".tar" ".Z" ".gz" ".taz" ".tgz" "tags")
;;  "If the given FILENAME includes extensions which are described in the form of
;;this variable, emacs regards it as a binary file.")
;;(defvar text-file-extentions
;;  nil
;;  "If the given FILENAME includes extensions which are described in the form of
;;this variable, emacs regards it as a text file.")


(defun find-buffer-file-type (filename)
  (let ((alist file-name-buffer-file-type-alist)
	(found nil)
	(code nil))
    (let ((case-fold-search filename-ignore-case))
      (setq filename (file-name-sans-versions filename))
      (while (and (not found) alist)
	(if (string-match (car (car alist)) filename)
	    (setq code (cdr (car alist))
		  found t))
	(setq alist (cdr alist))))
    (if found
	(cond((memq code '(nil t)) code)
	     ((and (symbolp code) (fboundp code))
	      (funcall code filename)))
      default-buffer-file-type)))

;;(defun find-buffer-file-type (filename)
;;  (let ((default default-buffer-file-type)
;;	(elist)
;;	(elt)
;;	(fname (file-name-sans-versions filename))
;;	(flen (length filename)))
;;    (setq elist (if default text-file-extentions binary-file-extentions))
;;    (if filename-ignore-case
;;	(catch 'exit
;;	  (while elist
;;	    (progn
;;	      (setq elt (car elist))
;;	      (if (equal (upcase elt)
;;			 (upcase (substring filename (- flen (length elt)))))
;;		  (throw 'exit (not default)))
;;	      (setq elist (cdr elist))))
;;	  default)
;;      (catch 'exit
;;	(while elist
;;	  (progn
;;	    (setq elt (car elist))
;;	    (if (equal elt
;;		       (substring filename (- flen (length elt))))
;;		(throw 'exit (not default)))
;;	    (setq elist (cdr elist))))
;;	default))))

(defun find-file-binary (filename) 
  "Visit file FILENAME and treat it as binary."
  (interactive "FFind file binary: ")
  (let ((file-name-buffer-file-type-alist '(("" . t))))
    (find-file filename)))

;;(defun find-file-binary (filename) 
;;  "Visit file FILENAME and treat it as binary."
;;  (interactive "FFind file binary: ")
;;  (let ((text-file-extentions nil)
;;	(default-buffer-file-type t))
;;    (find-file filename)))


(defun find-file-text (filename) 
  "Visit file FILENAME and treat it as a text file."
  (interactive "FFind file text: ")
  (let ((file-name-buffer-file-type-alist '(("" . nil))))
    (find-file filename)))

;;(defun find-file-text (filename) 
;;  "Visit file FILENAME and treat it as binary."
;;  (interactive "FFind file binary: ")
;;  (let ((binary-file-extentions nil)
;;	(default-buffer-file-type nil))
;;    (find-file filename)))


(defun find-file-not-found-set-buffer-file-type ()
  (save-excursion
    (set-buffer (current-buffer))
    (setq buffer-file-type (find-buffer-file-type (buffer-file-name))))
  nil)

;;; To set the default file type on new files.
(add-hook 'find-file-not-found-hooks 'find-file-not-found-set-buffer-file-type)


(setq completion-ignored-extensions
      (list ".o" ".elc" "~" ".BAK" ".bin" ".lbin" ".fasl"
	    ".dvi" ".toc" ".aux" ".a" ".ln"
	    ".lof" ".blg" ".bbl" ".glo" ".idx" ".lot" ".fmt"
	    ".x" ".r"
	    ".LB" ".PT4" ".GSX" ".SMD" ".DLB" ".DRX"
	    ".ZMD" ".ZPD" ".MDX" ".PDX" ".KDD" ".RCP" ".MID" ".MFF"
	    ".MAG" ".PIC" ".JPG" ".jpeg" ".TIF" ".tiff" ".GIF" ".Pi"))

(if (not (fboundp 'human-orig:file-readable-p))
    (fset 'human-orig:file-readable-p (symbol-function 'file-readable-p)))

(defun file-readable-p (filename)
  "Return t if file FILENAME exists and you can read it.
See also `file-exists-p' and `file-attributes'."
  (or (human-orig:file-readable-p filename)
      (file-directory-p filename)))


(defvar set-kanji-flag-hook nil
  "kanji-flag ZbgĂ΂tbNB")

(defun set-kanji-flag (value)
  "Jgobt@ kanji-flag  VALUE ɕύXB"
  (let ((oldvalue kanji-flag))
    (if value
	(let ((oldpt (point))
	      (modified (buffer-modified-p))
	      (buffer-undo-list nil)
	      (buffer-read-only nil))
	  (beginning-of-line)
	  (setq kanji-flag t)
	  (goto-char oldpt)
	  (insert " ")			; move the gap to valid position
	  (backward-delete-char 1)
	  (set-buffer-modified-p modified)
	  ;; in fact, all markers should be moved to valid position
	  )
      (setq kanji-flag nil))
    (run-hooks 'set-kanji-flag-hook)
    oldvalue))

(defun toggle-kanji-flag (&optional arg)
  "Shift-JIS {𗘗p / Ȃ؂ւB"
  (interactive "P")
  (set-kanji-flag
   (if (null arg)
       (not kanji-flag)
     (> arg 0)))
  (if (interactive-p)
      (redraw-display)))

(define-key ctl-x-map "\C-k" 'toggle-kanji-flag)
(or (assq 'kanji-flag minor-mode-alist)
    (nconc minor-mode-alist
	   (list '(kanji-flag " Kanji"))))


(setq file-name-handler-alist nil)	; inhibit ange-ftp

(autoload (quote display-time) "time2" "\
Display current time, load level, and mail flag in mode line of each buffer.
Updates automatically every minute.
If `display-time-day-and-date' is non-nil, the current day and date
are displayed as well.
After each update, `display-time-hook' is run with `run-hooks'." t nil)


;;; humanfns.el ends here
