;; Kinsoku shori for Egg
;; Copyright (C) 1992 Free Software Foundation, Inc.
;; This file is part of GNU Emacs.
;; This file contains Japanese characters.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; Coded by S.Tomura, Electrotechnical Lab. (tomura@etl.go.jp)

;;; 94.8     modified for Emacs 19.25 for Human68k by MM.

;;; 93.5.4   modified for Mule Ver.0.9.8 by K.Handa <handa@etl.go.jp>
;;;		   and K.Mugitani <a50350@sakura.kudpc.kyoto-u.ac.jp>
;;;	Kinsoku for GB and BIG5 added.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Change Log before Ver.0.9.7
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Modified for Mule 0.9.4 by K.Handa 92-May-2
;;;	Since syntax of multi-byte char is defined now,
;;;	check of English word is modified.
;;; Modified for Mule 0.9.0 by K.Handa 92-Mar-3
;;; Nemacs 3.2 created by S. Tomura 89-Nov-15
;;; Ver. 3.2  3.2 ΉɕύX
;;; Nemacs 3.0 created by S. Tomura 89-Mar-17
;;; Ver. 2.1a modified by S. Tomura 88-Nov-17
;;;           word̓rŕȂ悤ɏCB
;;; Ver. 2.1  modified by S. Tomura 88-Jun-24
;;;           kinsoku-shori moves the point <= fill-column + kinsoku-nobashi
;;; Nemacs V.2.1
;;; Ver. 1.1  modified by S. Tomura 88-Feb-29
;;;           Bug fix:  regexp-quote is used.
;;; Ver. 1.0  Created by S. Tomura
;;;           ֑@\񋟂B
;;;

(defvar japanese-kinsoku-version "3.21")
;;; Last modified date: Wed Nov 15 11:59:00 1989

;;; The followings must be merged into kanji.el
;;; patched by S.Tomura 87-Dec-7
;;;    JIS code̓ꕶ̈ꗗ\łBiˑj
;;;;     "@ABCDEFFGHIJKLMN"
;;;;   "OPQRSTUVWXYZ[\]^"
;;;;   "_`abcdefghijklmn"
;;;;   "opqrstuvwxyz{|}~"
;;;;   ""
;;;;   ""
;;;;     " "
;;;;     ""
;;;;   ""
;;;;     "ÃăŃƃǃȃɃʃ˃̃"
;;;;   "΃σЃу҃ӃԃՃ"
;;;;     "@ABCDEFGHIJKLMN"
;;;;   "OPQRSTUVWXYZ[\]^"
;;;;   "_`"
;;;;     "pqrstuvwxyz{|}~h
;;;;   ""
;;;;   ""
;;;;    OPQRSTUVWX`abcde
;;;;   ""
;;;;   "@BDFHb"

(defvar kinsoku-ascii t "Do kinsoku-shori for ASCII.")
(defvar kinsoku-jis t "Do kinsoku-shori for JISX0208.")

(defvar kinsoku-ascii-bol "!)-_~}]:;',.?" "BOL kinsoku for ASCII.")
(defvar kinsoku-ascii-eol "({[" "EOL kinsoku for ASCII.")
(defvar kinsoku-jis-bol
  (concat  "ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]"
	   "^_`abcdfhjlnprtvxz"
	   "@BDFHb")
  "BOL kinsoku for JISX0208.")
(defvar kinsoku-jis-eol
  "egikmoqsuwy"
  "EOL kinsoku for JISX0208.")

(defvar kinsoku-bol-chars 
  (concat
   (if kinsoku-ascii kinsoku-ascii-bol "")
   (if kinsoku-jis kinsoku-jis-bol ""))
  "s֑sȂׂĊ܂ޕw肷B
A string consisting of such characters that can't be beginning of line.")

(defvar  kinsoku-eol-chars
  (concat
   (if kinsoku-ascii kinsoku-ascii-eol "")
   (if kinsoku-jis kinsoku-jis-eol ""))
  "s֑sȂׂĊ܂ޕw肷B
A string consisting of such characters that can't be end of line.")

(defun kinsoku-char-bytes (ch)
  (if (> ch 256)
      2
    1))


(defun kinsoku-bol-p ()
  "pointŉsƍs֑ɐG邩ǂB
s֑kinsoku-bol-charsŎw肷B"
  (string-match "" "") ;;;regex comp̃ZbgłB
  (string-match (char-to-string (following-char))
		kinsoku-bol-chars))

(defun kinsoku-eol-p ()
  "pointŉsƍs֑ɐG邩ǂB
s֑kinsoku-eol-charsŎw肷B"
  (string-match "" "") ;;;regex comp̃ZbgłB
  (let ((ch (preceding-char)))
    (string-match (char-to-string ch)
		  kinsoku-eol-chars)))

(defvar kinsoku-nobashi-limit nil
  "֑ōsL΂ėǂpw肷B
񕉐ȊȌꍇ͖ӖB")

;;;###autoload
(defun kinsoku-shori ()
  "֑ɐGȂ_ֈړB
points֑ɐGꍇ͍sL΂āA֑ɐGȂ_TB
points֑ɐGꍇ͍sk߂āA֑ɐGȂ_TB
AsL΂pkinsoku-nobashi-limitzƁA
sk߂ċ֑ɐGȂ_TB"

  (let ((bol-kin nil) (eol-kin nil))
    (if (and (not (bolp))
	     (not (eolp))
	     (or (setq bol-kin (kinsoku-bol-p))
		 (setq eol-kin (kinsoku-eol-p))))
	(cond(bol-kin (kinsoku-shori-nobashi))
	     (eol-kin (kinsoku-shori-chizime))))))

;;;###autoload
(defun kinsoku-shori-nobashi ()
  "sL΂ċ֑ɐGȂ_ֈړB"
  (let ((max-column (+ fill-column 
		       (if (and (numberp kinsoku-nobashi-limit)
				(>= kinsoku-nobashi-limit 0))
			   kinsoku-nobashi-limit
			 10000)))  ;;; 10000͖̂łB
	ch1 ch2)			; 92.5.2 by K.Handa
    (while (and (<= (+ (current-column)
		       (char-width (setq ch1 (following-char))))
		    max-column)
		(not (bolp))
		(not (eolp))
		(or (kinsoku-eol-p)
		    (kinsoku-bol-p)
	            ;;; English word ̓rł͕ȂB
		    (and (< ch1 127)	; 92.5.2 by K.Handa
			 (< (setq ch2 (preceding-char)) 127)
			 (= ?w (char-syntax ch2))
			 (= ?w (char-syntax ch1)))))
      (forward-char))
    (if (or (kinsoku-eol-p) (kinsoku-bol-p))
	(kinsoku-shori-chizime))))

;;;###autoload
(defun kinsoku-shori-chizime ()
  "sk߂ċ֑ɐGȂ_ֈړB"
  (let (ch1 ch2)			; 92.5.2 by K.Handa
    (while (and (not (bolp))
		(not (eolp))
		(or (kinsoku-bol-p)
		    (kinsoku-eol-p)
		;;; English word ̓rł͕ȂB
		    (and		; 92.5.2 by K.Handa
		     (< (setq ch1 (following-char)) 127)
		     (< (setq ch2 (preceding-char)) 127)
		     (= ?w (char-syntax ch2))
		     (= ?w (char-syntax ch1)))))
      (backward-char))))
