;; x68-win.el --- keybinding for X68000 raw console terminal.

;;; This file is part of GNU Emacs for Human68K.
;;;
;;; GNU Emacs is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; GNU Emacs is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with GNU Emacs; see the file COPYING.  If not, write to
;;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;;; Code:

;; Command line args
;; -fg <color>	: default forecolor
;; -bg <color>  : default backcolor
;; -fn <font>   : default font

(setq command-switch-alist
      (append '(("-fg" .		x68-handle-forecolor)
		("-foreground" .	x68-handle-forecolor)
		("-bg" .		x68-handle-backcolor)
		("-background" .	x68-handle-backcolor)
		("-fn" .		x68-handle-font)
		("-font" .		x68-handle-font))
	      command-switch-alist))

(defun x68-handle-forecolor (switch)
  (let (col)
    (setq col (x68-convert-color-name (car x68-invocation-args))
	  x68-invocation-args (cdr x68-invocation-args))
    (if (numberp col)
	(setq x68-default-foreground-color col))))

(defun x68-handle-backcolor (arg)
  (let (col)
    (setq col (x68-convert-color-name (car x68-invocation-args))
	  x68-invocation-args (cdr x68-invocation-args))
    (if (numberp col)
	(setq x68-default-background-color col))))

(defun x68-handle-font (switch)
  (setq x68-default-font (car x68-invocation-args)
	x68-invocation-args (cdr x68-invocation-args)))

(defvar x68-invocation-args nil)

(defun x68-handle-args (args)
  (setq x68-invocation-args args
	args nil)
  (while x68-invocation-args
    (let* ((this-switch (car x68-invocation-args))
	   (aelt (assoc this-switch command-switch-alist)))
      (setq x68-invocation-args (cdr x68-invocation-args))
      (if aelt
	  (funcall (cdr aelt) this-switch)
	(setq args (cons this-switch args)))))
  (setq args (nreverse args)))
  

(setq command-line-args (x68-handle-args command-line-args))


;; modifier keys

(x68-set-modifier-key 'xf1 'alt)	; LALT
(x68-set-modifier-key 'xf2 'meta)	; LMETA
(x68-set-modifier-key 'xf3 'meta)	; RMETA
(x68-set-modifier-key 'xf4 'alt)	; RALT
(x68-set-modifier-key 'xf5 'control)
(x68-set-modifier-key 'opt.1 'super)
(x68-set-modifier-key 'opt.2 'hyper)


;; function keys

(define-key function-key-map [escape] 	[?\e])
(define-key global-map [backspace] 'backward-delete-char-untabify)
(define-key function-key-map [tab] 	[?\t])
(define-key function-key-map [return] 	[13])
;(define-key global-map [home]	'beginning-of-buffer)
(define-key function-key-map [delete] 	[127])
(define-key function-key-map [clear]	[11])
;(define-key function-key-map [kigo]	"...")
;(define-key function-key-map [toroku]	"...")
(define-key global-map [help]	'help-for-help)
(define-key function-key-map [break]	[7])
;(define-key function-key-map [copy]	"...")

(define-key global-map [M-next]		'scroll-other-window)
;(defun scroll-down-other-window (&optional n)
;  (interactive "P")
;  (scroll-other-window (if n (- 0 n) '-)))
(define-key global-map [M-prior]	'scroll-other-window-down)

(define-key global-map [f1]	'beginning-of-buffer)
(define-key global-map [f2]	'end-of-buffer)
(define-key global-map [f3]	'query-replace)
(define-key global-map [f4]	'isearch-forward)
(define-key global-map [f5]	'search-forward)
(define-key global-map [f6]	'set-mark-command)
(define-key global-map [f7]	'kill-region)
(define-key global-map [f8]	'kill-ring-save)
(define-key global-map [f9]	'yank)
(define-key global-map [f10]	'suspend-emacs)
(define-key global-map [S-f1]	'backward-page)
(define-key global-map [S-f2]	'forward-page)
(define-key global-map [S-f3]	'replace-string)
(define-key global-map [S-f4]	'isearch-backward)
(define-key global-map [S-f5]	'search-backward)
(define-key global-map [S-f6]	'set-mark-command)
(define-key global-map [S-f7]	'kill-region)
(define-key global-map [S-f8]	'kill-ring-save)
(define-key global-map [S-f9]	'yank)
(define-key global-map [S-f10]	'shell-command)

(defun eject-disk-a ()
  (interactive)
  (human-eject-disk "a:"))
(defun eject-disk-b ()
  (interactive)
  (human-eject-disk "b:"))
(defun eject-disk-c ()
  (interactive)
  (human-eject-disk "c:"))
(defun eject-disk-d ()
  (interactive)
  (human-eject-disk "d:"))
(define-key global-map [C-f1]	'eject-disk-a)
(define-key global-map [C-f2]	'eject-disk-b)
(define-key global-map [C-f3]	'eject-disk-c)
(define-key global-map [C-f4]	'eject-disk-d)
(define-key global-map [C-f5]	'human-flush-block-cache)
(define-key global-map [C-f7]	'x68-toggle-function-key-label)
(define-key global-map [C-f8]	'x68-toggle-mouse-cursor)
(define-key global-map [C-f9]	'scroll-bar-mode)

(define-key function-key-map [M-escape] [?\M-\e])
(define-key function-key-map [M-backspace] [?\M-\d])
(define-key function-key-map [M-tab] [?\M-\t])
(define-key function-key-map [M-return] [?\M-\015])
(define-key function-key-map [M-delete] [?\M-\d])
(define-key function-key-map [M-clear] [?\M-\013])


(put 'backspace 'ascii-character 127)
(put 'delete 'ascii-character 127)
(put 'tab 'ascii-character ?\t)
(put 'clear 'ascii-character 12)
(put 'return 'ascii-character 13)
(put 'escape 'ascii-character ?\e)


(x68-label-function-key 1 " 擪s" "Oy[W" "eject a:")
(x68-label-function-key 2 " ŏIs" "y[W" "eject b:")
(x68-label-function-key 3 "mFu" "ꊇu" "eject c:")
(x68-label-function-key 4 "i-search" "i-search" "eject d:")
(x68-label-function-key 5 "O" "" "  SYNC")
(x68-label-function-key 6 " }[N" " }[N")
(x68-label-function-key 7 "   " "   " "  FKEY")
(x68-label-function-key 8 "   " "   " " MOUSE")
(x68-label-function-key 9 "\t" "\t" "SCRL-BAR")
(x68-label-function-key 10 "suspend" "shellcom")



(x68-toggle-mouse-cursor 1)
(x68-toggle-function-key-label 1)



(setq x68-font-path (list data-directory "/usr/lib/fonts/"))

(require 'x68-faces)
(defalias 'face-initialize 'x68-face-initialize)



(require 'x68-mouse)

(define-key global-map [down-mouse-1] 'mouse-drag-region)
(global-set-key [mouse-1]	'mouse-set-point)
(global-set-key [drag-mouse-1]	'mouse-set-region)

;; These are tested for in mouse-drag-region.
(global-set-key [double-mouse-1] 'mouse-set-point)
(global-set-key [triple-mouse-1] 'mouse-set-point)

(global-set-key [mouse-2]	'mouse-yank-at-click)
(global-set-key [S-mouse-2]	'mouse-save-then-kill)

;; Replaced with dragging mouse-1
;; (global-set-key [S-mouse-1]	'mouse-set-mark)

(global-set-key [mode-line mouse-1] 'mouse-select-window)
(global-set-key [mode-line drag-mouse-1] 'mouse-select-window)
(global-set-key [mode-line down-mouse-1] 'mouse-drag-mode-line)
(global-set-key [mode-line mouse-2] 'mouse-delete-window)
(global-set-key [mode-line S-mouse-2] 'mouse-delete-other-windows)
(global-set-key [mode-line C-mouse-2] 'mouse-split-window-horizontally)

(global-set-key [vertical-scroll-bar down-mouse-1] 'scroll-bar-scroll)

;;; X11 łƎ悤ȑ쐫𓾂ꍇÂPsāA
;;; ̂Ts𐶂ĂBAX11 ł̒{^ (mouse-2) 
;;; @\́ASHIFT + {^Ɋ蓖Ă܂B
;;(global-set-key [vertical-scroll-bar mouse-1] 'scroll-bar-scroll-up)
;;(global-set-key [vertical-scroll-bar drag-mouse-1] 'scroll-bar-scroll-up)
;;(global-set-key [vertical-scroll-bar mouse-2] 'scroll-bar-scroll-down)
;;(global-set-key [vertical-scroll-bar drag-mouse-2] 'scroll-bar-scroll-down)
;;(global-set-key [vertical-scroll-bar S-down-mouse-1] 'scroll-bar-drag)

(global-set-key [vertical-scroll-bar C-mouse-2] 'mouse-split-window-vertically)
(global-set-key [vertical-line C-mouse-2] 'mouse-split-window-vertically)


;;
;; for compatibility...
(defun x-get-resource (a b) nil)
(defun x-display-grayscale-p (&optional a) nil)
(defun x-display-color-p nil t)
(defun window-frame (win) 0)

(defun x68-set-pallette-code (a b)
  (error "Obsolete function: x68-set-pallet-code"))

;;; x68-win.el ends here
