;;; x68-faces.el --- Lisp interface to the c "face" structure for x68-win

;; Copyright (C) 1992, 1993, 1994 Free Software Foundation, Inc. / MINOURA Makoto

;; This file is part of GNU Emacs for Human68K.

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;;; Commentary:

;; Modified by MM based on faces.el

;;; Code:

(defvar x68-default-foreground-color 65342)
(defvar x68-default-background-color 20480)
(defvar x68-default-font "rom16")



;; compatibility...
(defalias 'frame-face-alist 'x68-frame-face-alist)
(defalias 'set-frame-face-alist 'x68-set-frame-face-alist)
(defalias 'make-face-internal 'x68-make-face-internal)
(defalias 'set-face-attribute-internal 'x68-set-face-attribute-internal)
(defalias 'internal-next-face-id 'x68-internal-next-face-id)



;;;; Functions for manipulating face vectors.

;;; A face vector is a vector of the form:
;;;    [face NAME ID FONT FOREGROUND BACKGROUND BACKGROUND-PIXMAP UNDERLINE BOLD]

;;; Type checkers.
(defsubst internal-facep (x)
  (and (vectorp x) (= (length x) 9) (eq (aref x 0) 'face)))

(defmacro internal-check-face (face)
  (` (while (not (internal-facep (, face)))
       (setq (, face) (signal 'wrong-type-argument (list 'internal-facep (, face)))))))

;;; Accessors.
(defsubst face-name (face)
  "Return the name of face FACE."
  (aref (internal-get-face face) 1))

(defsubst face-id (face)
  "Return the internal ID number of face FACE."
  (aref (internal-get-face face) 2))

(defsubst face-font (face &optional frame)
  "Return the font name of face FACE, or nil if it is unspecified.
The optional argument FRAME is omitted."
  (aref (internal-get-face face) 3))

(defsubst face-foreground (face &optional frame)
  "Return the foreground color name of face FACE, or nil if unspecified.
The optional argument FRAME is omitted."
  (aref (internal-get-face face) 4))

(defsubst face-background (face &optional frame)
  "Return the background color name of face FACE, or nil if unspecified.
The optional argument FRAME is omitted."
  (aref (internal-get-face face) 5))

;;(defsubst face-background-pixmap (face &optional frame)
;; "Return the background pixmap name of face FACE, or nil if unspecified.
;;If the optional argument FRAME is given, report on face FACE in that frame.
;;Otherwise report on the defaults for face FACE (for new frames)."
;; (aref (internal-get-face face frame) 6))

(defsubst face-underline-p (face &optional frame)
 "Return t if face FACE is underlined.
The optional argument FRAME is omitted."
 (aref (internal-get-face face) 7))

(defsubst x68-face-bold-p (face &optional frame)
 "Return t if face FACE is bolded (i.e. double-typed).
The optional argument FRAME is omitted."
 (aref (internal-get-face face) 8))


;; x68-win color name interface

(defvar x68-color-name-code-alist
	(list (cons "snow" 65534)
	      (cons "ghost white" 65534)
	      (cons "ghostwhite" 65534)
	      (cons "white smoke" 63420)
	      (cons "whitesmoke" 63420)
	      (cons "gainsboro" 57078)
	      (cons "floral white" 65532)
	      (cons "floralwhite" 65532)
	      (cons "old lace" 63480)
	      (cons "oldlace" 63480)
	      (cons "linen" 63480)
	      (cons "antique white" 61428)
	      (cons "antiquewhite" 61428)
	      (cons "papaya whip" 61428)
	      (cons "papayawhip" 61428)
	      (cons "blanched almond" 61426)
	      (cons "blanchedalmond" 61426)
	      (cons "bisque" 59376)
	      (cons "peach puff" 57326)
	      (cons "peachpuff" 57326)
	      (cons "navajo white" 57322)
	      (cons "navajowhite" 57322)
	      (cons "moccasin" 59372)
	      (cons "cornsilk" 65526)
	      (cons "ivory" 65532)
	      (cons "lemon chiffon" 65522)
	      (cons "lemonchiffon" 65522)
	      (cons "seashell" 63482)
	      (cons "honeydew" 65468)
	      (cons "mint cream" 65470)
	      (cons "mintcream" 65470)
	      (cons "azure" 65470)
	      (cons "alice blue" 65470)
	      (cons "aliceblue" 65470)
	      (cons "lavender" 59198)
	      (cons "lavender blush" 63484)
	      (cons "lavenderblush" 63484)
	      (cons "misty rose" 59384)
	      (cons "mistyrose" 59384)
	      (cons "white" 65534)
	      (cons "black" 0)
	      (cons "dark slate gray" 18770)
	      (cons "darkslategray" 18770)
	      (cons "dark slate grey" 18770)
	      (cons "darkslategrey" 18770)
	      (cons "dim gray" 27482)
	      (cons "dimgray" 27482)
	      (cons "dim grey" 27482)
	      (cons "dimgrey" 27482)
	      (cons "slate gray" 33700)
	      (cons "slategray" 33700)
	      (cons "slate grey" 33700)
	      (cons "slategrey" 33700)
	      (cons "light slate gray" 35750)
	      (cons "lightslategray" 35750)
	      (cons "light slate grey" 35750)
	      (cons "lightslategrey" 35750)
	      (cons "gray" 48622)
	      (cons "grey" 48622)
	      (cons "light grey" 54964)
	      (cons "lightgrey" 54964)
	      (cons "light gray" 54964)
	      (cons "lightgray" 54964)
	      (cons "midnight blue" 6364)
	      (cons "midnightblue" 6364)
	      (cons "navy" 32)
	      (cons "navy blue" 32)
	      (cons "navyblue" 32)
	      (cons "cornflower blue" 37690)
	      (cons "cornflowerblue" 37690)
	      (cons "dark slate blue" 14946)
	      (cons "darkslateblue" 14946)
	      (cons "slate blue" 23410)
	      (cons "slateblue" 23410)
	      (cons "medium slate blue" 27642)
	      (cons "mediumslateblue" 27642)
	      (cons "light slate blue" 29758)
	      (cons "lightslateblue" 29758)
	      (cons "medium blue" 50)
	      (cons "mediumblue" 50)
	      (cons "royal blue" 27192)
	      (cons "royalblue" 27192)
	      (cons "blue" 62)
	      (cons "dodger blue" 37118)
	      (cons "dodgerblue" 37118)
	      (cons "deep sky blue" 47166)
	      (cons "deepskyblue" 47166)
	      (cons "sky blue" 52282)
	      (cons "skyblue" 52282)
	      (cons "light sky blue" 52286)
	      (cons "lightskyblue" 52286)
	      (cons "steel blue" 33324)
	      (cons "steelblue" 33324)
	      (cons "light steel blue" 50614)
	      (cons "lightsteelblue" 50614)
	      (cons "light blue" 56696)
	      (cons "lightblue" 56696)
	      (cons "powder blue" 58808)
	      (cons "powderblue" 58808)
	      (cons "pale turquoise" 60794)
	      (cons "paleturquoise" 60794)
	      (cons "dark turquoise" 51252)
	      (cons "darkturquoise" 51252)
	      (cons "medium turquoise" 53874)
	      (cons "mediumturquoise" 53874)
	      (cons "turquoise" 57908)
	      (cons "cyan" 63550)
	      (cons "light cyan" 65342)
	      (cons "lightcyan" 65342)
	      (cons "cadet blue" 39656)
	      (cons "cadetblue" 39656)
	      (cons "medium aquamarine" 52010)
	      (cons "mediumaquamarine" 52010)
	      (cons "aquamarine" 64500)
	      (cons "dark green" 24576)
	      (cons "darkgreen" 24576)
	      (cons "dark olive green" 27274)
	      (cons "darkolivegreen" 27274)
	      (cons "dark sea green" 48226)
	      (cons "darkseagreen" 48226)
	      (cons "sea green" 35156)
	      (cons "seagreen" 35156)
	      (cons "medium sea green" 45532)
	      (cons "mediumseagreen" 45532)
	      (cons "light sea green" 45354)
	      (cons "lightseagreen" 45354)
	      (cons "pale green" 64742)
	      (cons "palegreen" 64742)
	      (cons "spring green" 63518)
	      (cons "springgreen" 63518)
	      (cons "lawn green" 64448)
	      (cons "lawngreen" 64448)
	      (cons "green" 63488)
	      (cons "chartreuse" 64448)
	      (cons "medium spring green" 63526)
	      (cons "mediumspringgreen" 63526)
	      (cons "green yellow" 64842)
	      (cons "greenyellow" 64842)
	      (cons "lime green" 51596)
	      (cons "limegreen" 51596)
	      (cons "yellow green" 52428)
	      (cons "yellowgreen" 52428)
	      (cons "forest green" 35080)
	      (cons "forestgreen" 35080)
	      (cons "olive drab" 35656)
	      (cons "olivedrab" 35656)
	      (cons "dark khaki" 46554)
	      (cons "darkkhaki" 46554)
	      (cons "khaki" 59298)
	      (cons "pale goldenrod" 61290)
	      (cons "palegoldenrod" 61290)
	      (cons "light goldenrod yellow" 65524)
	      (cons "lightgoldenrodyellow" 65524)
	      (cons "light yellow" 65528)
	      (cons "lightyellow" 65528)
	      (cons "yellow" 65472)
	      (cons "gold" 55232)
	      (cons "light goldenrod" 57184)
	      (cons "lightgoldenrod" 57184)
	      (cons "goldenrod" 42696)
	      (cons "dark goldenrod" 34242)
	      (cons "darkgoldenrod" 34242)
	      (cons "rosy brown" 36322)
	      (cons "rosybrown" 36322)
	      (cons "indian red" 24150)
	      (cons "indianred" 24150)
	      (cons "saddle brown" 17476)
	      (cons "saddlebrown" 17476)
	      (cons "sienna" 21770)
	      (cons "peru" 34382)
	      (cons "burlywood" 48864)
	      (cons "beige" 63414)
	      (cons "wheat" 57260)
	      (cons "sandy brown" 42904)
	      (cons "sandybrown" 42904)
	      (cons "tan" 46754)
	      (cons "chocolate" 28294)
	      (cons "firebrick" 9608)
	      (cons "brown" 11530)
	      (cons "dark salmon" 38750)
	      (cons "darksalmon" 38750)
	      (cons "salmon" 34780)
	      (cons "light salmon" 42974)
	      (cons "lightsalmon" 42974)
	      (cons "orange" 42944)
	      (cons "dark orange" 36800)
	      (cons "darkorange" 36800)
	      (cons "coral" 32724)
	      (cons "light coral" 34720)
	      (cons "lightcoral" 34720)
	      (cons "tomato" 26576)
	      (cons "orange red" 18368)
	      (cons "orangered" 18368)
	      (cons "red" 1984)
	      (cons "hot pink" 28652)
	      (cons "hotpink" 28652)
	      (cons "deep pink" 6116)
	      (cons "deeppink" 6116)
	      (cons "pink" 51186)
	      (cons "light pink" 47088)
	      (cons "lightpink" 47088)
	      (cons "pale violet red" 30436)
	      (cons "palevioletred" 30436)
	      (cons "maroon" 13720)
	      (cons "medium violet red" 5664)
	      (cons "mediumvioletred" 5664)
	      (cons "violet red" 9892)
	      (cons "violetred" 9892)
	      (cons "magenta" 2046)
	      (cons "violet" 34682)
	      (cons "plum" 42742)
	      (cons "orchid" 30452)
	      (cons "medium orchid" 22004)
	      (cons "mediumorchid" 22004)
	      (cons "dark orchid" 13554)
	      (cons "darkorchid" 13554)
	      (cons "dark violet" 1204)
	      (cons "darkviolet" 1204)
	      (cons "blue violet" 11384)
	      (cons "blueviolet" 11384)
	      (cons "purple" 9532)
	      (cons "medium purple" 29878)
	      (cons "mediumpurple" 29878)
	      (cons "thistle" 48886)
	      (cons "snow1" 65534)
	      (cons "snow2" 61306)
	      (cons "snow3" 52850)
	      (cons "snow4" 35938)
	      (cons "seashell1" 63482)
	      (cons "seashell2" 59254)
	      (cons "seashell3" 50798)
	      (cons "seashell4" 33888)
	      (cons "antiquewhite1" 61430)
	      (cons "antiquewhite2" 57202)
	      (cons "antiquewhite3" 50796)
	      (cons "antiquewhite4" 33886)
	      (cons "bisque1" 59376)
	      (cons "bisque2" 55148)
	      (cons "bisque3" 46694)
	      (cons "bisque4" 31834)
	      (cons "peachpuff1" 57326)
	      (cons "peachpuff2" 53098)
	      (cons "peachpuff3" 44644)
	      (cons "peachpuff4" 29784)
	      (cons "navajowhite1" 57322)
	      (cons "navajowhite2" 53096)
	      (cons "navajowhite3" 46690)
	      (cons "lemonchiffon1" 65522)
	      (cons "lemonchiffon2" 61294)
	      (cons "lemonchiffon3" 52840)
	      (cons "lemonchiffon4" 35932)
	      (cons "cornsilk1" 65526)
	      (cons "cornsilk2" 61298)
	      (cons "cornsilk3" 52844)
	      (cons "cornsilk4" 35934)
	      (cons "ivory1" 65532)
	      (cons "ivory2" 61304)
	      (cons "ivory3" 52848)
	      (cons "ivory4" 35936)
	      (cons "honeydew1" 65468)
	      (cons "honeydew2" 61240)
	      (cons "honeydew3" 52784)
	      (cons "honeydew4" 35872)
	      (cons "lavenderblush1" 63484)
	      (cons "lavenderblush2" 59256)
	      (cons "lavenderblush3" 50800)
	      (cons "lavenderblush4" 33888)
	      (cons "mistyrose1" 59384)
	      (cons "mistyrose2" 55156)
	      (cons "mistyrose3" 46700)
	      (cons "mistyrose4" 31838)
	      (cons "azure1" 65470)
	      (cons "azure2" 61242)
	      (cons "azure3" 52786)
	      (cons "azure4" 35874)
	      (cons "slateblue1" 27710)
	      (cons "slateblue2" 25594)
	      (cons "slateblue3" 23410)
	      (cons "slateblue4" 14882)
	      (cons "royalblue1" 29310)
	      (cons "royalblue2" 27194)
	      (cons "royalblue3" 23026)
	      (cons "royalblue4" 16674)
	      (cons "blue1" 62)
	      (cons "blue2" 58)
	      (cons "blue3" 50)
	      (cons "blue4" 34)
	      (cons "dodgerblue1" 37118)
	      (cons "dodgerblue2" 33018)
	      (cons "dodgerblue3" 28914)
	      (cons "dodgerblue4" 18594)
	      (cons "steelblue1" 47934)
	      (cons "steelblue2" 43770)
	      (cons "steelblue3" 37490)
	      (cons "steelblue4" 24994)
	      (cons "deepskyblue1" 47166)
	      (cons "deepskyblue2" 45114)
	      (cons "deepskyblue3" 38962)
	      (cons "deepskyblue4" 26658)
	      (cons "skyblue1" 52286)
	      (cons "skyblue2" 50170)
	      (cons "skyblue3" 41842)
	      (cons "skyblue4" 29282)
	      (cons "lightskyblue1" 58814)
	      (cons "lightskyblue2" 54586)
	      (cons "lightskyblue3" 46194)
	      (cons "lightskyblue4" 31522)
	      (cons "slategray1" 58942)
	      (cons "slategray2" 54778)
	      (cons "slategray3" 46322)
	      (cons "slategray4" 31586)
	      (cons "lightsteelblue1" 59006)
	      (cons "lightsteelblue2" 54778)
	      (cons "lightsteelblue3" 46386)
	      (cons "lightsteelblue4" 31586)
	      (cons "lightblue1" 60926)
	      (cons "lightblue2" 56762)
	      (cons "lightblue3" 50418)
	      (cons "lightblue4" 33634)
	      (cons "lightcyan1" 65342)
	      (cons "lightcyan2" 61114)
	      (cons "lightcyan3" 52658)
	      (cons "lightcyan4" 35810)
	      (cons "paleturquoise1" 65022)
	      (cons "paleturquoise2" 60794)
	      (cons "paleturquoise3" 52402)
	      (cons "paleturquoise4" 35618)
	      (cons "cadetblue1" 62718)
	      (cons "cadetblue2" 58490)
	      (cons "cadetblue3" 50162)
	      (cons "cadetblue4" 33442)
	      (cons "turquoise1" 61502)
	      (cons "turquoise2" 57402)
	      (cons "turquoise3" 49202)
	      (cons "turquoise4" 32802)
	      (cons "cyan1" 63550)
	      (cons "cyan2" 59450)
	      (cons "cyan3" 51250)
	      (cons "cyan4" 34850)
	      (cons "darkslategray1" 64702)
	      (cons "darkslategray2" 60538)
	      (cons "darkslategray3" 52210)
	      (cons "darkslategray4" 35490)
	      (cons "aquamarine1" 64500)
	      (cons "aquamarine2" 60336)
	      (cons "aquamarine3" 52010)
	      (cons "aquamarine4" 35356)
	      (cons "darkseagreen1" 65072)
	      (cons "darkseagreen2" 60844)
	      (cons "darkseagreen3" 52454)
	      (cons "darkseagreen4" 35674)
	      (cons "seagreen1" 64166)
	      (cons "seagreen2" 60004)
	      (cons "seagreen3" 51744)
	      (cons "palegreen1" 64742)
	      (cons "palegreen2" 60580)
	      (cons "palegreen3" 52190)
	      (cons "springgreen1" 63518)
	      (cons "springgreen2" 59420)
	      (cons "springgreen3" 51224)
	      (cons "darkolivegreen1" 65116)
	      (cons "darkolivegreen2" 60890)
	      (cons "khaki1" 63458)
	      (cons "khaki2" 59232)
	      (cons "khaki3" 50780)
	      (cons "lightgoldenrod1" 61410)
	      (cons "lightgoldenrod2" 57184)
	      (cons "lightgoldenrod3" 48732)
	      (cons "lightyellow1" 65528)
	      (cons "lightyellow2" 61300)
	      (cons "lightyellow3" 52844)
	      (cons "lightyellow4" 35934)
	      (cons "rosybrown1" 51184)
	      (cons "rosybrown2" 46956)
	      (cons "rosybrown3" 40550)
	      (cons "rosybrown4" 27738)
	      (cons "indianred1" 28634)
	      (cons "burlywood1" 55270)
	      (cons "burlywood2" 51044)
	      (cons "burlywood3" 44638)
	      (cons "wheat1" 59374)
	      (cons "wheat2" 57194)
	      (cons "wheat3" 48740)
	      (cons "wheat4" 31832)
	      (cons "salmon1" 36826)
	      (cons "lightsalmon1" 42974)
	      (cons "lightsalmon2" 38748)
	      (cons "deeppink1" 6116)
	      (cons "deeppink2" 5986)
	      (cons "deeppink3" 5724)
	      (cons "hotpink1" 28652)
	      (cons "hotpink2" 28520)
	      (cons "hotpink3" 26212)
	      (cons "hotpink4" 15448)
	      (cons "pink1" 47088)
	      (cons "pink2" 44910)
	      (cons "pink3" 38502)
	      (cons "pink4" 25690)
	      (cons "lightpink1" 45038)
	      (cons "lightpink2" 42858)
	      (cons "lightpink3" 36452)
	      (cons "lightpink4" 23640)
	      (cons "palevioletred1" 34794)
	      (cons "palevioletred2" 32614)
	      (cons "palevioletred3" 28258)
	      (cons "maroon1" 14316)
	      (cons "maroon2" 14184)
	      (cons "maroon3" 11876)
	      (cons "violetred1" 16356)
	      (cons "violetred2" 16226)
	      (cons "violetred3" 13918)
	      (cons "magenta1" 2046)
	      (cons "magenta2" 1914)
	      (cons "magenta3" 1650)
	      (cons "magenta4" 1122)
	      (cons "orchid1" 34814)
	      (cons "orchid2" 32634)
	      (cons "orchid3" 28274)
	      (cons "orchid4" 17506)
	      (cons "plum1" 49150)
	      (cons "plum2" 44922)
	      (cons "plum3" 38514)
	      (cons "plum4" 25698)
	      (cons "mediumorchid1" 26430)
	      (cons "mediumorchid2" 24250)
	      (cons "mediumorchid3" 21938)
	      (cons "mediumorchid4" 13282)
	      (cons "darkorchid1" 15870)
	      (cons "darkorchid2" 15802)
	      (cons "darkorchid3" 13554)
	      (cons "darkorchid4" 9058)
	      (cons "purple1" 13566)
	      (cons "purple2" 11450)
	      (cons "purple3" 9202)
	      (cons "purple4" 6818)
	      (cons "mediumpurple1" 34174)
	      (cons "mediumpurple2" 31994)
	      (cons "mediumpurple3" 27762)
	      (cons "mediumpurple4" 17122)
	      (cons "thistle1" 59390)
	      (cons "thistle2" 55162)
	      (cons "thistle3" 46706)
	      (cons "thistle4" 31842)
	      (cons "gray0" 0)
	      (cons "grey0" 0)
	      (cons "gray1" 0)
	      (cons "grey1" 0)
	      (cons "gray2" 0)
	      (cons "grey2" 0)
	      (cons "gray3" 2114)
	      (cons "grey3" 2114)
	      (cons "gray4" 2114)
	      (cons "grey4" 2114)
	      (cons "gray5" 2114)
	      (cons "grey5" 2114)
	      (cons "gray6" 2114)
	      (cons "grey6" 2114)
	      (cons "gray7" 4228)
	      (cons "grey7" 4228)
	      (cons "gray8" 4228)
	      (cons "grey8" 4228)
	      (cons "gray9" 4228)
	      (cons "grey9" 4228)
	      (cons "gray10" 6342)
	      (cons "grey10" 6342)
	      (cons "gray11" 6342)
	      (cons "grey11" 6342)
	      (cons "gray12" 6342)
	      (cons "grey12" 6342)
	      (cons "gray13" 8456)
	      (cons "grey13" 8456)
	      (cons "gray14" 8456)
	      (cons "grey14" 8456)
	      (cons "gray15" 8456)
	      (cons "grey15" 8456)
	      (cons "gray16" 10570)
	      (cons "grey16" 10570)
	      (cons "gray17" 10570)
	      (cons "grey17" 10570)
	      (cons "gray18" 10570)
	      (cons "grey18" 10570)
	      (cons "gray19" 12684)
	      (cons "grey19" 12684)
	      (cons "gray20" 12684)
	      (cons "grey20" 12684)
	      (cons "gray21" 12684)
	      (cons "grey21" 12684)
	      (cons "gray22" 14798)
	      (cons "grey22" 14798)
	      (cons "gray23" 14798)
	      (cons "grey23" 14798)
	      (cons "gray24" 14798)
	      (cons "grey24" 14798)
	      (cons "gray25" 16912)
	      (cons "grey25" 16912)
	      (cons "gray26" 16912)
	      (cons "grey26" 16912)
	      (cons "gray27" 16912)
	      (cons "grey27" 16912)
	      (cons "gray28" 16912)
	      (cons "grey28" 16912)
	      (cons "gray29" 19026)
	      (cons "grey29" 19026)
	      (cons "gray30" 19026)
	      (cons "grey30" 19026)
	      (cons "gray31" 19026)
	      (cons "grey31" 19026)
	      (cons "gray32" 21140)
	      (cons "grey32" 21140)
	      (cons "gray33" 21140)
	      (cons "grey33" 21140)
	      (cons "gray34" 21140)
	      (cons "grey34" 21140)
	      (cons "gray35" 23254)
	      (cons "grey35" 23254)
	      (cons "gray36" 23254)
	      (cons "grey36" 23254)
	      (cons "gray37" 23254)
	      (cons "grey37" 23254)
	      (cons "gray38" 25368)
	      (cons "grey38" 25368)
	      (cons "gray39" 25368)
	      (cons "grey39" 25368)
	      (cons "gray40" 25368)
	      (cons "grey40" 25368)
	      (cons "gray41" 27482)
	      (cons "grey41" 27482)
	      (cons "gray42" 27482)
	      (cons "grey42" 27482)
	      (cons "gray43" 27482)
	      (cons "grey43" 27482)
	      (cons "gray44" 29596)
	      (cons "grey44" 29596)
	      (cons "gray45" 29596)
	      (cons "grey45" 29596)
	      (cons "gray46" 29596)
	      (cons "grey46" 29596)
	      (cons "gray47" 31710)
	      (cons "grey47" 31710)
	      (cons "gray48" 31710)
	      (cons "grey48" 31710)
	      (cons "gray49" 31710)
	      (cons "grey49" 31710)
	      (cons "gray50" 31710)
	      (cons "grey50" 31710)
	      (cons "gray51" 33824)
	      (cons "grey51" 33824)
	      (cons "gray52" 33824)
	      (cons "grey52" 33824)
	      (cons "gray53" 33824)
	      (cons "grey53" 33824)
	      (cons "gray54" 35938)
	      (cons "grey54" 35938)
	      (cons "gray55" 35938)
	      (cons "grey55" 35938)
	      (cons "gray56" 35938)
	      (cons "grey56" 35938)
	      (cons "gray57" 38052)
	      (cons "grey57" 38052)
	      (cons "gray58" 38052)
	      (cons "grey58" 38052)
	      (cons "gray59" 38052)
	      (cons "grey59" 38052)
	      (cons "gray60" 40166)
	      (cons "grey60" 40166)
	      (cons "gray61" 40166)
	      (cons "grey61" 40166)
	      (cons "gray62" 40166)
	      (cons "grey62" 40166)
	      (cons "gray63" 42280)
	      (cons "grey63" 42280)
	      (cons "gray64" 42280)
	      (cons "grey64" 42280)
	      (cons "gray65" 42280)
	      (cons "grey65" 42280)
	      (cons "gray66" 44394)
	      (cons "grey66" 44394)
	      (cons "gray67" 44394)
	      (cons "grey67" 44394)
	      (cons "gray68" 44394)
	      (cons "grey68" 44394)
	      (cons "gray69" 46508)
	      (cons "grey69" 46508)
	      (cons "gray70" 46508)
	      (cons "grey70" 46508)
	      (cons "gray71" 46508)
	      (cons "grey71" 46508)
	      (cons "gray72" 48622)
	      (cons "grey72" 48622)
	      (cons "gray73" 48622)
	      (cons "grey73" 48622)
	      (cons "gray74" 48622)
	      (cons "grey74" 48622)
	      (cons "gray75" 48622)
	      (cons "grey75" 48622)
	      (cons "gray76" 50736)
	      (cons "grey76" 50736)
	      (cons "gray77" 50736)
	      (cons "grey77" 50736)
	      (cons "gray78" 50736)
	      (cons "grey78" 50736)
	      (cons "gray79" 52850)
	      (cons "grey79" 52850)
	      (cons "gray80" 52850)
	      (cons "grey80" 52850)
	      (cons "gray81" 52850)
	      (cons "grey81" 52850)
	      (cons "gray82" 54964)
	      (cons "grey82" 54964)
	      (cons "gray83" 54964)
	      (cons "grey83" 54964)
	      (cons "gray84" 54964)
	      (cons "grey84" 54964)
	      (cons "gray85" 57078)
	      (cons "grey85" 57078)
	      (cons "gray86" 57078)
	      (cons "grey86" 57078)
	      (cons "gray87" 57078)
	      (cons "grey87" 57078)
	      (cons "gray88" 59192)
	      (cons "grey88" 59192)
	      (cons "gray89" 59192)
	      (cons "grey89" 59192)
	      (cons "gray90" 59192)
	      (cons "grey90" 59192)
	      (cons "gray91" 61306)
	      (cons "grey91" 61306)
	      (cons "gray92" 61306)
	      (cons "grey92" 61306)
	      (cons "gray93" 61306)
	      (cons "grey93" 61306)
	      (cons "gray94" 63420)
	      (cons "grey94" 63420)
	      (cons "gray95" 63420)
	      (cons "grey95" 63420)
	      (cons "gray96" 63420)
	      (cons "grey96" 63420)
	      (cons "gray97" 63420)
	      (cons "grey97" 63420)
	      (cons "gray98" 65534)
	      (cons "grey98" 65534)
	      (cons "gray99" 65534)
	      (cons "grey99" 65534)
	      (cons "gray100" 65534)
	      (cons "grey100" 65534)
	      (cons "dark grey" 44394)
	      (cons "darkgrey" 44394)
	      (cons "dark gray" 44394)
	      (cons "darkgray" 44394)
	      (cons "dark magenta" 1122)
	      (cons "darkmagenta" 1122)
	      (cons "dark red" 1088)
	      (cons "darkred" 1088)
	      (cons "light green" 60580)
	      (cons "lightgreen" 60580)))

(defun x68-convert-color-name (name)
  (let ((code))
    (cond ((numberp name)
	   name)
	  ((setq code (cdr (assoc (downcase name) x68-color-name-code-alist)))
	   code)
	  (t
	   (string-to-number name)))))



;;; Mutators.

(defun x-resolve-font-name (pattern &optional face frame)
  "Dummy function"
  pattern)

(defsubst set-face-font (face font &optional frame)
  "Change the font of face FACE to FONT (a string).
The optional argument FRAME is omitted."
  (interactive (internal-face-interactive "font"))
  (if (stringp font) (setq font (x-resolve-font-name font face frame)))
  (internal-set-face-1 face 'font font 3))

(defsubst set-face-foreground (face color &optional frame)
  "Change the foreground color of face FACE to COLOR (a string).
The optional argument FRAME is omitted."
  (interactive (internal-face-interactive "foreground"))
  (internal-set-face-1 face 'foreground color 4 frame))

(defsubst set-face-background (face color &optional frame)
  "Change the background color of face FACE to COLOR (a string).
The optional argument FRAME is omitted."
  (interactive (internal-face-interactive "background"))
  (internal-set-face-1 face 'background color 5 frame))

(defsubst set-face-underline-p (face underline-p &optional frame)
  "Specify whether face FACE is underlined.  (Yes if UNDERLINE-P is non-nil.)
The optional argument FRAME is omitted."
  (interactive (internal-face-interactive "underline-p" "underlined"))
  (internal-set-face-1 face 'underline underline-p 7 frame))

(defsubst x68-set-face-bold-p (face bold-p &optional frame)
  "Specify whether face FACE is bolded (i.e. double-typed).
(Yes if BOLD-P is non-nil.)
The optional argument FRAME is omitted."
  (interactive (internal-face-interactive "bold-p" "bold"))
  (internal-set-face-1 face 'bold bold-p 8 frame))




;;;; Associating face names (symbols) with their face vectors.

(defun face-list ()
  "Returns a list of all defined face names."
  (mapcar 'car (frame-face-alist)))

(defun internal-find-face (name &optional frame)
  "Retrieve the face named NAME.  Return nil if there is no such face.
Optional argument FRAME is omitted."
  (if (symbolp name)
      (cdr (assq name
		 (frame-face-alist nil)))
    (internal-check-face name)
    name))

(defun internal-get-face (name &optional frame)
  "Retrieve the face named NAME; error if there is none.
The Optional argument FRAME is omitted."
  (or (internal-find-face name)
      (internal-check-face name)))


(defun x68-font-name (filename)
  (let ((font (file-name-nondirectory filename)))
    (if filename-ignore-case (setq font (downcase font)))
    (if (string-match "\\.fon" font)
	(substring font 0 -4)
      font)))

(defun internal-set-face-1 (face name value index &optional frame)
  (if (and (eq name 'foreground)
	   (null value))
      (setq value x68-default-foreground-color))
  (if (and (eq name 'background)
	   (null value))
      (setq value x68-default-background-color))
  (if (or (eq name 'foreground)
	  (eq name 'background))
      (setq value (x68-convert-color-name value)))
  (if (and (eq name 'font)
	   (null value))
      (setq value x68-default-font))
  (let ((inhibit-quit t))
    (set-face-attribute-internal (face-id face) name value))
  (aset (internal-get-face face)
	index
	(if (eq name 'face) (x68-font-name value) value)))


(defun read-face-name (prompt)
  (let (face)
    (while (= (length face) 0)
      (setq face (completing-read prompt
				  (mapcar '(lambda (x) (list (symbol-name x)))
					  (face-list))
				  nil t)))
    (intern face)))

(defun internal-face-interactive (what &optional bool)
  (let* ((fn (intern (concat "face-" what)))
	 (prompt (concat "Set " what " of face"))
	 (face (read-face-name (concat prompt ": ")))
	 (default (if (fboundp fn)
		      (or (funcall fn face)
			  (funcall fn 'default))))
	 (value (if bool
		    (y-or-n-p (concat "Should face " (symbol-name face)
				      " be " bool "? "))
		  (if (numberp default)
		      (setq default (number-to-string default)))
		  (read-string (concat prompt " " (symbol-name face) " to: ")
			       default))))
    (list face (if (equal value "") nil value))))


(defun make-face (name)
  "Define a new FACE on all frames.  
You can modify the font, color, etc of this face with the set-face- functions.
If the face already exists, it is unmodified."
  (interactive "SMake face: ")
  (or (internal-find-face name)
      (let ((face (make-vector 9 nil)))
	(aset face 0 'face)
	(aset face 1 name)
	(let* ((inhibit-quit t)
	       (id (internal-next-face-id)))
	  (make-face-internal id)
	  (aset face 2 id)
	  (if (= id 0)
	      (progn
		(aset face 3 x68-default-font)
		(aset face 4 x68-default-foreground-color)
		(aset face 5 x68-default-background-color)
		(set-face-attribute-internal 0 'font x68-default-font)
		(set-face-attribute-internal 0 'foreground x68-default-foreground-color)
		(set-face-attribute-internal 0 'background x68-default-background-color)))
	  (set-frame-face-alist (cons (cons name (copy-sequence face))
				      (frame-face-alist)))
	  (if (/= id 0)
	      (progn
		(set-face-font name (face-font 'default))
		(set-face-foreground name (face-foreground 'default))
		(set-face-background name (face-background 'default))
		(set-face-underline-p name (face-underline-p 'default))
		(x68-set-face-bold-p name (x68-face-bold-p 'default)))))
	face))
  name)

(defun copy-face (old-face new-face &optional frame new-frame)
  "Define a face just like OLD-FACE, with name NEW-FACE.
If NEW-FACE already exists as a face, it is modified to be like OLD-FACE.
If it doesn't already exist, it is created.
The optional arguments FRAME and NEW-FRAME are omitted."
  (let ((inhibit-quit t))
    (setq old-face (internal-get-face old-face))
    (setq new-face (or (internal-find-face new-face)
		       (make-face new-face)))
    (condition-case nil
	;; A face that has a global symbolic font modifier such as `bold'
	;; might legitimately get an error here.
	;; Use the frame's default font in that case.
	(set-face-font new-face (face-font old-face))
      (error
       (set-face-font new-face x68-default-font)))
    (set-face-foreground new-face (face-foreground old-face))
    (set-face-background new-face (face-background old-face))
    (set-face-underline-p new-face (face-underline-p old-face))
    (x68-set-face-bold-p new-face (x68-face-bold-p old-face)))
  new-face)

(defun face-equal (face1 face2 &optional frame)
  "True if the faces FACE1 and FACE2 display in the same way."
  (setq face1 (internal-get-face face1)
	face2 (internal-get-face face2))
  (and (equal (face-foreground face1) (face-foreground face2))
       (equal (face-background face1) (face-background face2))
       (equal (face-font face1) (face-font face2))
       (eq (face-underline-p face1) (face-underline-p face2))
       (eq (x68-face-bold-p face1) (x68-face-bold-p face2))
       ))

(defun face-differs-from-default-p (face &optional frame)
  "True if face FACE displays differently from the default face.
A face is considered to be ``the same'' as the default face if it is 
actually specified in the same way (equivalent fonts, etc) or if it is 
fully unspecified, and thus inherits the attributes of any face it 
is displayed on top of.
The optional argument FRAME is omitted."
  (let ((default (internal-get-face 'default)))
    (setq face (internal-get-face face))
    (not (and (or (equal (face-foreground default)
			 (face-foreground face))
		  (null (face-foreground face)))
	      (or (equal (face-background default)
			 (face-background face))
		  (null (face-background face)))
	      (or (equal (face-font default) (face-font face))
		  (null (face-font face)))
	      (equal (face-underline-p default)
		     (face-underline-p face))
	      (equal (x68-face-bold-p default)
		     (x68-face-bold-p face))
	      ))))


(defun invert-face (face &optional frame)
  "Swap the foreground and background colors of face FACE."
  (interactive (list (read-face-name "Invert face: ")))
  (setq face (internal-get-face face))
  (let ((fg (face-foreground face))
	(bg (face-background face)))
    (set-face-foreground face bg)
    (set-face-background face fg))
  face)


(defun internal-try-face-font (face font &optional frame)
  "Like set-face-font, but returns nil on failure instead of an error."
  (condition-case ()
      (set-face-font face font)
    (error nil)))


;;; non-X-specific interface

(defun make-face-bold (face &optional frame noerror)
  "Make the font of the given face be bold, if possible.  
If NOERROR is non-nil, return nil on failure."
  (interactive (list (read-face-name "Make which face bold: ")))
  (let ((ofont (face-font face)))
    (cond ((and (string-equal ofont x68-default-font)
		(not (x68-face-bold-p face)))
	   (or (internal-try-face-font face "bold")
	       (x68-set-face-bold-p face t)))
	  ((and (string-equal ofont "italic")
		(not (x68-face-bold-p face)))
	   (or (internal-try-face-font face "bold-italic")
	       (x68-set-face-bold-p face t)))
	  ((not (x68-face-bold-p face))
	   (x68-set-face-bold-p face t))
	  (t
	   (if (not noerror)
	       (error "No bold version of %S" ofont)
	     nil)))))

(defun make-face-italic (face &optional frame noerror)
  "Make the font of the given face be italic, if possible.  
If NOERROR is non-nil, return nil on failure."
  (interactive (list (read-face-name "Make which face italic: ")))
  (let ((ofont (face-font face)))
    (cond ((and (string-equal ofont x68-default-font)
		(not (x68-face-bold-p face)))
	   (or (internal-try-face-font face "italic")
	       (if (not noerror) (error "No italic version of %S" ofont)
		   nil)))
	  ((and (string-equal ofont x68-default-font)
		(x68-face-bold-p face))
	   (or (internal-try-face-font face "bold-italic")
	       (and (internal-try-face-font face "italic")
		    (x68-set-face-bold-p face t))
	       (if (not noerror) (error "No italic version of %S" ofont)
		   nil)))
	  (t
	   (if (not noerror)
	       (error "No italic version of %S" (face-font face))
	     nil)))))

(defun make-face-bold-italic (face &optional frame noerror)
  "Make the font of the given face be bold and italic, if possible.  
If NOERROR is non-nil, return nil on failure."
  (interactive (list (read-face-name "Make which face bold-italic: ")))
  (let ((ofont (face-font face)))
    (cond ((and (string-equal ofont x68-default-font)
		(not (x68-face-bold-p face)))
	   (or (internal-try-face-font face "bold-italic")
	       (and (internal-try-face-font face "italic")
		    (x68-set-face-bold-p face t))
	       (if (not noerror)
		   (error "No bold italic version of %S" ofont)
		 nil)))
	  (t
	   (if (not noerror)
	       (error "No bold italic version of %S" ofont)
	     nil)))))

;;(defun make-face-unbold (face &optional frame noerror)
;;  "Make the font of the given face be non-bold, if possible.  
;;If NOERROR is non-nil, return nil on failure."
;;  (interactive (list (read-face-name "Make which face non-bold: ")))
;;  (if (and (eq frame t) (listp (face-font face t)))
;;      (set-face-font face (if (memq 'italic (face-font face t))
;;			      '(italic) nil)
;;		     t)
;;    (let ((ofont (face-font face frame))
;;	  font font1)
;;      (if (null frame)
;;	  (let ((frames (frame-list)))
;;	    ;; Make this face unbold in global-face-data.
;;	    (make-face-unbold face t noerror)
;;	    ;; Make this face unbold in each frame.
;;	    (while frames
;;	      (make-face-unbold face (car frames) noerror)
;;	      (setq frames (cdr frames))))
;;	(setq face (internal-get-face face frame))
;;	(setq font1 (or (face-font face frame)
;;			(face-font face t)))
;;	(if (listp font1)
;;	    (setq font1 nil))
;;	(setq font1 (or font1
;;			(face-font 'default frame)
;;			(cdr (assq 'font (frame-parameters frame)))))
;;	(setq font (x-make-font-unbold font1))
;;	(if font (internal-try-face-font face font frame)))
;;      (or (not (equal ofont (face-font face)))
;;	  (and (not noerror)
;;	       (error "No unbold version of %S" font1))))))

;;(defun make-face-unitalic (face &optional frame noerror)
;;  "Make the font of the given face be non-italic, if possible.  
;;If NOERROR is non-nil, return nil on failure."
;;  (interactive (list (read-face-name "Make which face non-italic: ")))
;;  (if (and (eq frame t) (listp (face-font face t)))
;;      (set-face-font face (if (memq 'bold (face-font face t))
;;			      '(bold) nil)
;;		     t)
;;    (let ((ofont (face-font face frame))
;;	  font font1)
;;      (if (null frame)
;;	  (let ((frames (frame-list)))
;;	    ;; Make this face unitalic in global-face-data.
;;	    (make-face-unitalic face t noerror)
;;	    ;; Make this face unitalic in each frame.
;;	    (while frames
;;	      (make-face-unitalic face (car frames) noerror)
;;	      (setq frames (cdr frames))))
;;	(setq face (internal-get-face face frame))
;;	(setq font1 (or (face-font face frame)
;;			(face-font face t)))
;;	(if (listp font1)
;;	    (setq font1 nil))
;;	(setq font1 (or font1
;;			(face-font 'default frame)
;;			(cdr (assq 'font (frame-parameters frame)))))
;;	(setq font (x-make-font-unitalic font1))
;;	(if font (internal-try-face-font face font frame)))
;;      (or (not (equal ofont (face-font face)))
;;	  (and (not noerror)
;;	       (error "No unitalic version of %S" font1))))))



(defvar list-faces-sample-text
  (if (boundp 'kanji-flag)
      "abcdefghIJKLMNOP ̓jzwg    0123 ABCD "
    "abcdefghijklmnopqrstuvwxyz ABCDEFGHIJKLMNOPQRSTUVWXYZ")
  "*Text string to display as the sample text for `list-faces-display'.")

;; The name list-faces would be more consistent, but let's avoid a conflict
;; with Lucid, which uses that name differently.
(defun list-faces-display ()
  "List all faces, using the same sample text in each.
The sample text is a string that comes from the variable
`list-faces-sample-text'.

It is possible to give a particular face name different appearances in
different frames.  This command shows the appearance in the
selected frame."
  (interactive)
  (let ((faces (sort (face-list) (function string-lessp)))
	(face nil))
    (with-output-to-temp-buffer "*Faces*"
      (save-excursion
	(set-buffer standard-output)
	(setq truncate-lines t)
	(while faces
	  (setq face (car faces))
	  (setq faces (cdr faces))
	  (insert (format "%25s " (symbol-name face)))
	  (let ((beg (point)))
	    (insert list-faces-sample-text)
	    (insert "\n")
	    (put-text-property beg (1- (point)) 'face face)
	    ;; If the sample text has multiple lines, line up all of them.
	    (goto-char beg)
	    (forward-line 1)
	    (while (not (eobp))
	      (insert "                          ")
	      (forward-line 1))))
	(goto-char (point-min))))))


;;; x-win conpatibility

(defun set-foreground-color (color)
  "Set the foreground color of the selected frame to COLOR.
When called interactively, prompt for the name of the color to use."
  (interactive "sColor: ")
  (set-face-foreground 'default color))

(defun set-background-color (color)
  "Set the background color of the selected frame to COLOR.
When called interactively, prompt for the name of the color to use."
  (interactive "sColor: ")
  (set-face-background 'default color))



;;; Make the standard faces.
;;; The C code knows the default and modeline faces as faces 0 and 1,
;;; so they must be the first two faces made.
(defun x68-face-initialize ()
  (if (eq window-system 'x68)
      (progn
	(make-face 'default)
	(make-face 'modeline)
	(make-face 'highlight)

	;; These aren't really special in any way, but they're nice to have around.

	(make-face 'bold)
	(make-face 'italic)
	(make-face 'bold-italic)
	(make-face 'region)
	(make-face 'secondary-selection)
	(make-face 'underline)

	(setq region-face (face-id 'region))

	;; Specify the global properties of these faces
	;; so they will come out right on new frames.

	(make-face-bold 'bold nil t)
	(make-face-italic 'italic nil t)
	(make-face-bold-italic 'bold-italic nil t)

;;	(set-face-background 'highlight '("darkseagreen2" "green" t) t)
	(set-face-background 'highlight "darkseagreen2")
;;	(set-face-background 'region '("gray" underline) t)
	(set-face-background 'region "gray")
;;	(set-face-background 'secondary-selection '("paleturquoise" "green" t) t)
	(set-face-background 'secondary-selection "paleturquoise" t)
;;	(set-face-background 'modeline '(t) t)
	(set-face-foreground 'highlight "black")
	(set-face-foreground 'region "black")
	(set-face-foreground 'secondary-selection "black" t)
	(invert-face 'modeline)
	(set-face-underline-p 'underline t)))
  )



(provide 'x68-faces)

;;; x68-faces.el ends here
